//
//  Joueur.swift
//  Swiftement Pokemon
//
//  Created by Onoulade on 16/01/2022.
//

import Foundation

/*
 On crée un protocole Capacite qui permettra de regrouper les sorts et les attaques facilement
 */

protocol Capacite {
    var nom: String { get set }
    var element: Element? { get set }
    var recharges: Int { get set }
    var rechargesMax: Int { get set }
    
    func utiliser(avec joueur: Pokemon, sur ennemi: Pokemon) -> Bool
}

/*
 La classe joueur sera utilisée par le joueur et l'adversaire pour un code simplifié
 */

class Joueur {
    
    // Le joueur est l'adversaire
    var adversaire: Bool
    
    // Tous les pokémons sélectionnés par le joueur
    var pokemons: [Pokemon] = []
    
    // Le pokémon sélectionné actuellement, peut être nul
    var pokemonSelectionne: Pokemon?
    
    // La liste de toutes les capacités du pokémon actuel
    var capacites: [Capacite] {
        get {
            var capacites: [Capacite] = []
            if let pokemon = pokemonSelectionne {
                for capacite in pokemon.capacites {
                    capacites.append(capacite)
                    
                }
            }
            
            return capacites
        }
    }
    
    // La liste des capacités disponibles du pokémon actuel
    var capacitesDisponibles:[Capacite] {
        get {
            var capacites: [Capacite] = []
            for capacite in self.capacites {
                if capacite.recharges > 0 {
                    capacites.append(capacite)
                }
            }
            
            return capacites
        }
    }
    
    // La liste des attaques disponibles du pokémon actuel
    var attaquesDisponibles: [Attaque] {
        get {
            var attaques: [Attaque] = []
            for capacite in capacitesDisponibles {
                if capacite is Attaque  {
                    attaques.append(capacite as! Attaque)
                }
            }
            return attaques
        }
    }
    
    // La liste des sorts disponibles du pokémon actuel
    var sortsDisponibles: [Sort] {
        get {
            var attaques: [Sort] = []
            for capacite in capacitesDisponibles {
                if capacite is Sort  {
                    attaques.append(capacite as! Sort)
                }
            }
            return attaques
        }
    }
    
    // La liste de tous les pokémons non-étourdis
    var pokemonsDisponibles: [Pokemon] {
        get {
            var disponibles: [Pokemon] = []
            for pokemon in pokemons {
                if pokemon.vivant {
                    disponibles.append(pokemon)
                }
            }
            
            return disponibles
        }
    }
    
    // Le pokémon actuel, cette variable se met à jour automatiquement si le pokémon est étourdit ou non
    var pokemonActuel: Pokemon? {
        get {
            if pokemonSelectionne?.vivant == false {
                pokemonSelectionne = nil
            }
            return pokemonSelectionne
        }
        set {
            pokemonSelectionne = newValue
        }
    }
    
    init(adversaire:Bool = false) {
        self.adversaire = adversaire
    }
    
    // Sélection d'un nouveau pokémon pour le joueur
    func choisirPokemon() {
        if !adversaire {
            print("Qui souhaitez-vous envoyer au combat ?")
            
            for (i, pokemon) in pokemonsDisponibles.enumerated() {
                print(i + 1, ")", pokemon.nom)
            }
            
            let selection = readSelection(max: pokemonsDisponibles.count-1)
            pokemonActuel = pokemonsDisponibles[selection]
            
            print(pokemonActuel!.nom,"à l'attaque !")
        }
        else {
            let tirage = random(min: 0, max: pokemonsDisponibles.count - 1)
            pokemonActuel = pokemonsDisponibles[tirage]
            
            print("Votre adversaire envoie", pokemonSelectionne!.nom)
        }
        
        print("")
    }
    
    // Ajout d'un pokémon aux pokémons sélectionnés (début de partie)
    func add(pokemon: Pokemon) {
        pokemons.append(pokemon)
    }
}
