//
//  main.swift
//  Swiftement Pokemon
//
//  Created by Onoulade on 04/12/2021.
//

import Foundation

// - MARK: Fonctions de base

func random() -> Float {
    return Float(arc4random()) / 0xFFFFFFFF
}

func random(min: Int, max: Int) -> Int {
    let floatAdd = Float(max - min)
    return Int(floor(random() * floatAdd) + Float(min))
}

func select(_ i: Int, joueur: Joueur) {
    joueur.add(pokemon: pool[i])
    pool.remove(at: i)
}

func readSelection(max: Int) -> Int {
    
    /*
     Cette fonction va nous permettre de lire une entrée utilisateur et vérifier
     qu'il s'agisse bien d'un nombre avant de retourner un résultat.
     Si l'entrée est invalide, on en redemande une nouvelle sans avoir à quitter la fonction
     */
    
    // Pour ça on crée une variable optionnelle nulle
    var selection: Int?
    
    // On détermine une boucle qui s'exécutera tant que la variable est nulle
    while selection == nil {
        
        // On lit l'entrée (au format String)
        let entree: String? = readLine()
        
        /*
         On convertit cette entrée au format Int puis on y soustrait 1
         Si l'entrée n'est pas un nombre, on utilise zéro par défaut qui ne peut pas un choix fonctionnel dans notre jeu
         Comme tous les choix proposés commenceront à 1, on soustrait 1 pour s'assurer de retourner une valeur
         utilisable comme clé dans un tableau (1 devient 0, 2 devient, 1, etc...)
         */
        let entreeInt = (Int(entree ?? "") ?? 0) - 1
        
        // On s'assure que la clé soit bien comprise entre 0 et le nombre max de choix
        if entreeInt < 0 || entreeInt > max {
            print("Sélection invalide")
            
            // Si ce n'est pas le cas, on recommence la boucle
            continue
            
        }
        
        // Si on n'a pas recommencé la boucle, on enregistre notre variable ce qui aura
        // pour effet de quitter la boucle
        selection = entreeInt
    }
    
    return selection!
}

// -MARK: Initialisation des variables

let pokemons: [Pokemon] = [
    .init(nom: "Pikachu", ptsVie: 25, ptsDefense: 1, capacites: [Attaque.bouleDeFeu, Attaque.couteaux, Sort.soin], element: .ciseaux),
    .init(nom: "Racaillou", ptsVie: 15, ptsDefense: 3, capacites: [Attaque.gravillons, Attaque.coupDeBoule, Sort.intimidation], element: .pierre),
    .init(nom: "Rondoudou", ptsVie: 30, ptsDefense: 0, capacites: [Attaque.rosier, Attaque.bouleDeFeu, Sort.soin], element: .feuille),
    .init(nom: "Salamèche", ptsVie: 20, ptsDefense: 2, capacites: [Attaque.bouleDeFeu, Attaque.couteaux, Attaque.coupDeBoule], element: .ciseaux),
    .init(nom: "Magicarpe", ptsVie: 25, ptsDefense: 1, capacites: [Attaque.rosier, Sort.intimidation, Sort.soin], element: .feuille),
    .init(nom: "Tadmorv", ptsVie: 20, ptsDefense: 2, capacites: [Attaque.gravillons, Attaque.coupDeBoule, Sort.soin], element: .pierre)
]

var pool: [Pokemon] = pokemons

let joueur = Joueur()
let adversaire = Joueur(adversaire: true)

var jeuEnCours = true

// - MARK: Boucle principale
// C'est parti ! Voici notre boucle principale de jeu.
while jeuEnCours {
    
    // On imprime une ligne vide à chaque début de boucle pour séparer les différents
    // tours du jeu de manière à ce que ce soit un peu plus lisible
    print("")
    
    /*
     On commence par vérifier que le joueur ait bien 3 pokémons attribués.
     Si ce n'est pas le cas c'est forcément que le jeu n'a pas commencé et qu'il faut les attribuer
     c'est donc la première chose que l'on fait, on sait qu'une fois qu'ils seront enregistrés
     ce code ne s'exécutera plus
     */
    if joueur.pokemons.count < 3 {
        
        print("Choisissez vos Pokémons : ")
        for (i, pokemon) in pool.enumerated() {
            // On affiche l'index + 1 pour commencer le compte à 1 ce qui est plus facilement lisible
            print(i + 1, pokemon.nom)
        }
        
        let pokemonChoisi = readSelection(max: pool.count-1)
        
        select(pokemonChoisi, joueur:joueur)
        
        // Une fois qu'un pokémon a été choisit, on sort de la boucle et on recommence si nécessaire
        continue
    }
    
    /*
     Une fois que le joueur a sélectionné 3 pokémons, on vérifie que l'adversaire en ait aussi.
     Le code s'exécute après car le joueur a la priorité sur le choix des pokémons.
     Cette fois-ci on vérifie pas qu'il y en ait suffisamment, on sait que si il n'y en a pas
     alors il faut les choisir, et vu que l'IA ne les choisit pas les uns après les autres
     avec une entrée utilisateur, on sait que si il y en a au moins 1 alors il y sont tous
     */
    if adversaire.pokemons.count == 0 {
        print("")
        for _ in 0..<3 {
            let tirage = random(min: 0, max: pool.count-1)
            print("L'adversaire choisit le pokémon", pool[tirage].nom)
            select(tirage, joueur:adversaire)
            
        }
        
        print("Que le combat commence !")
        print("")
    }
    
    /*
     On va vérifier que le joueur ait bien un pokémon sélectionné, si il n'en a pas c'est
     que son pokémon actuel est étoudit et qu'il va devoir en sélectionner un autre
     */
    
    if joueur.pokemonActuel == nil {
        if joueur.pokemonsDisponibles.count == 0 {
            // Bien sûr si il n'a plus de pokémons disponibles, il a perdu et c'est la fin du jeu
            print("Vous n'avez plus de pokémons en état de combattre, vous avez perdu !")
            jeuEnCours = false
            continue
        }
        
        joueur.choisirPokemon()
    }
    
    // Même chose pour l'adversaire
    if adversaire.pokemonActuel == nil {
        if adversaire.pokemonsDisponibles.count == 0 {
            print("Votre adversaire n'a plus de pokémons, vous avez gagné !")
            jeuEnCours = false
            continue
        }
        
        adversaire.choisirPokemon()
    }
    
    // On sécurise nos variables afin de ne pas travailler avec des variables optionnelles de partout
    guard let pokemonJoueur = joueur.pokemonActuel, let pokemonAdversaire = adversaire.pokemonActuel else {
        continue
    }
    
    /*
     Ici on va devoir faire une nouvelle boucle pour s'assurer que le joueur effectue une action.
     Il est possible qu'il entre une réponse illisible ou essaie de faire une action qu'il
     n'est pas autorisé à faire, alors on ne continue pas tant qu'une action jouable à été effectuée
     */
    
    var ajoue = false
    
    while (!ajoue) {
        let viePokemonString = "(" + String(pokemonJoueur.ptsVie) + "/" + String(pokemonJoueur.maxVie) + ")"
        print("A vous de jouer :", viePokemonString)
        
        // On liste toutes les capacités possibles au joueur
        let capacites = joueur.capacites
        for (i, capacite) in capacites.enumerated() {
            let rechargesString = "(" + String(capacite.recharges) + "/" + String(capacite.rechargesMax) + ")"
            print(i+1, ")", capacite.nom, rechargesString)
        }
        print("")
        
        // On lui propose également de pouvir changer de pokémon
        print(capacites.count+1, ") Changer de pokemon")
        
        let selection = readSelection(max: capacites.count)
        
        if selection < capacites.count {
            // On tente d'exécuter l'action demandée par le joueur, si les conditions sont validées
            // alors on pourra sortir de la boucle
            ajoue = capacites[selection].utiliser(avec: pokemonJoueur, sur: pokemonAdversaire)
        }
        else {
            joueur.choisirPokemon()
            ajoue = true
        }
    }
    
    // Si l'action qui vient d'être effectuée a étourdit le pokémon adverse, on propose au joueur de changer
    
    if adversaire.pokemonActuel == nil  {
        print("Souhaitez-vous changer de Pokémon ?")
        print("1) Non")
        print("2) Oui")
        let selection = readSelection(max: 1)
        if selection == 1 {
            joueur.choisirPokemon()
        }
        continue
    }
    
    /*
     C'est parti pour le tour de l'adversaire, ici l'IA n'est pas remarquable mais elle permet
     d'avoir un semblant d'intelligence au jeu
     */
    
    // Tout d'abord, si le pokémon actuel n'a plus de capacités disponible et qu'un autre est disponible, on change
    
    if (adversaire.capacitesDisponibles.count == 0 && adversaire.pokemonsDisponibles.count > 1) {
        adversaire.choisirPokemon()
        continue
    }
    
    // On va ensuite tirer un éventuel lancé de sort
    
    var sortAUtiliser: Sort? = nil
    
    for sort in adversaire.sortsDisponibles {
        /*
         Si c'est un  sort de soin et que le pokémon a moins de la moitié de ses points de vie,
         on l'utilise d'office.
         OU
         Si c'est un sort défensif, on effectue un tirage au sort d'1/3
         */
        if (sort.vieRestauree > 0 && pokemonAdversaire.ptsVie < pokemonAdversaire.maxVie / 2) || (sort.defenseEnnemieDiminuee > 0 && random(min: 0, max: 3) == 0) {
            sortAUtiliser = sort
            continue
        }
    }
    
    // Si un sort a été sélectionné, on le joue
    if let sort = sortAUtiliser {
        _ = sort.utiliser(avec: pokemonAdversaire, sur: pokemonJoueur)
        continue
    }
    
    // Si aucun sort n'a été joué mais qu'une attaque n'esr disponible, on change de pokémon
    
    if adversaire.attaquesDisponibles.count == 0 {
        adversaire.choisirPokemon()
    }
    else {
        
        // Sinon on tire au sort une attaque et on la lance
        
        let tirage = random(min: 0, max: adversaire.attaquesDisponibles.count - 1)
        _ = adversaire.attaquesDisponibles[tirage].utiliser(avec: pokemonAdversaire, sur: pokemonJoueur)
    }
    
    
    
    
}

